#! /usr/bin/env python3
# -*- coding: utf-8 -*-
""" Construct boomerang fraction sequences for a list of input values.

In this module, the boomerang function includes:
    
    - Shortcut inversion check: If a single inversion, followed by one 
      or more subtractions, is sufficient to return the current 
      sequence value to 1, then an inversion is performed immediately.
      This results in shorter sequences for some fractions.

    - Inversion limit check: Exceeding this limit indicates that the
      algorithm implemented in boomerang may not be able to construct a
      sequence that returns to 1.
"""


from fractions import Fraction


ADDITION_OPERATION = "\u2192"
INVERSION_OPERATION = "\u21bb"
OUTPUT_TEMPLATE = "{}/{}\n{}\n"
SEQUENCE_DELIMITER = " "
FAILURE_MESSAGE = "Failed"


def boomerang(fraction, limit=100):
    """Construct and return a boomerang fraction sequence.
    
    Arguments:
        fraction -- Additive term in boomerang fraction sequence.
        limit    -- Maximum number of inversions that will be performed
                    before the function terminates unsuccessfully. A
                    value of 0 or None disables the limit check.
    Return value:
        List of interleaved values and operations.  If the first item
        in the list is not 1, then the function has terminated without
        successfully completing a boomerang fraction sequence.
    """
    value = Fraction(1)
    sequence = [value]
    inversion_count = 0
    while True:
        value -= fraction
        sequence = [value, ADDITION_OPERATION] + sequence
        if value == 1:
            break
        if (
            value <= fraction  # Subtraction not possible; invert.
            or (
                value < 1
                and (1 / value - 1) % fraction == 0
            )  # Shortcut available; invert.
        ):
            if limit and inversion_count >= limit:  # No more inversions.
                break
            value = 1 / value
            inversion_count += 1
            sequence = [value, INVERSION_OPERATION] + sequence
    return sequence


if __name__ == '__main__':
    for m, n in [(1, 2), (2, 3), (1, 4), (4, 5), (2, 5), (7, 9), (2, 7)]:
        fraction = Fraction(m, n)
        sequence = boomerang(fraction)
        if sequence[0] == 1:
            print(OUTPUT_TEMPLATE.format(
                m, n, SEQUENCE_DELIMITER.join(str(s) for s in sequence)))
        else:
            print(OUTPUT_TEMPLATE.format(m, n, FAILURE_MESSAGE))
